# Stubs for fractions
# See https://docs.python.org/3/library/fractions.html
#
# Note: these stubs are incomplete. The more complex type
# signatures are currently omitted. Also see numbers.pyi.

from typing import Optional, TypeVar, Union, overload, Any
from numbers import Real, Integral, Rational
from decimal import Decimal
import sys

_ComparableNum = Union[int, float, Decimal, Real]


@overload
def gcd(a: int, b: int) -> int: ...
@overload
def gcd(a: Integral, b: int) -> Integral: ...
@overload
def gcd(a: int, b: Integral) -> Integral: ...
@overload
def gcd(a: Integral, b: Integral) -> Integral: ...


class Fraction(Rational):
    @overload
    def __init__(self,
                 numerator: Union[int, Rational] = ...,
                 denominator: Optional[Union[int, Rational]] = ...,
                 *,
                 _normalize: bool = ...) -> None: ...
    @overload
    def __init__(self, value: float, *, _normalize: bool = ...) -> None: ...
    @overload
    def __init__(self, value: Decimal, *, _normalize: bool = ...) -> None: ...
    @overload
    def __init__(self, value: str, *, _normalize: bool = ...) -> None: ...

    @classmethod
    def from_float(cls, f: float) -> Fraction: ...
    @classmethod
    def from_decimal(cls, dec: Decimal) -> Fraction: ...
    def limit_denominator(self, max_denominator: int = ...) -> Fraction: ...

    @property
    def numerator(self) -> int: ...
    @property
    def denominator(self) -> int: ...

    def __add__(self, other): ...
    def __radd__(self, other): ...
    def __sub__(self, other): ...
    def __rsub__(self, other): ...
    def __mul__(self, other): ...
    def __rmul__(self, other): ...
    def __truediv__(self, other): ...
    def __rtruediv__(self, other): ...
    if sys.version_info < (3, 0):
        def __div__(self, other): ...
        def __rdiv__(self, other): ...
    def __floordiv__(self, other) -> int: ...
    def __rfloordiv__(self, other) -> int: ...
    def __mod__(self, other): ...
    def __rmod__(self, other): ...
    def __divmod__(self, other): ...
    def __rdivmod__(self, other): ...
    def __pow__(self, other): ...
    def __rpow__(self, other): ...

    def __pos__(self) -> Fraction: ...
    def __neg__(self) -> Fraction: ...
    def __abs__(self) -> Fraction: ...
    def __trunc__(self) -> int: ...
    if sys.version_info >= (3, 0):
        def __floor__(self) -> int: ...
        def __ceil__(self) -> int: ...
        def __round__(self, ndigits: Optional[Any] = ...): ...

    def __hash__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...
    def __lt__(self, other: _ComparableNum) -> bool: ...
    def __gt__(self, other: _ComparableNum) -> bool: ...
    def __le__(self, other: _ComparableNum) -> bool: ...
    def __ge__(self, other: _ComparableNum) -> bool: ...
    if sys.version_info >= (3, 0):
        def __bool__(self) -> bool: ...
    else:
        def __nonzero__(self) -> bool: ...

    # Not actually defined within fractions.py, but provides more useful
    # overrides
    @property
    def real(self) -> Fraction: ...
    @property
    def imag(self) -> Fraction: ...
    def conjugate(self) -> Fraction: ...



# Getting variables to be shared between modules can be difficult.
# This class exists so that modules from r.py can talk to modules in the rp_ptpython package.
# Essentially, this class is th middle-man. The other modules will both read and write to this module.
# This module is pretty hacky...but that's OK. It doesn't really respect nested pseudoterminals though, so if we were to make multiple async pseudoterminals this module couldn't exist.
globa={}
last_assignable_comm=None
pseudo_terminal_level=0
rp_evaluator=lambda x:None# Returns strings!! UNLESS the second argument is True
current_input_text=""
rp_pt_VARS=""
python_input_buffers={}# str ⟶ str
rp_evaluator_mem=None
parenthesized_line=""
try_eval_mem_text=""
ans=None
clipboard_text=""#Should be in sync with pyperclip if its not broken
current_candidates=[]#Is overwritten in completer.py of rp_ptpython, and used in space-completion of callables
enable_space_autocompletions=[]#Make this list non-empty BY MUTATING IT to enable space completions (make bool(enable_space_autocompletions) True
dont_erase_buffer_on_enter=[]#as opposed to [True], which is truthy
completion_style=['good']#should be either ['good'] or ['fast']
debug_height=20#The height of the debugger toolbar
successful_commands=[]



# Stubs for distutils.archive_util

from typing import Optional


def make_archive(base_name: str, format: str, root_dir: Optional[str] = ...,
                 base_dir: Optional[str] = ..., verbose: int = ...,
                 dry_run: int = ...) -> str: ...
def make_tarball(base_name: str, base_dir: str, compress: Optional[str] = ...,
                 verbose: int = ..., dry_run: int = ...) -> str: ...
def make_zipfile(base_name: str, base_dir: str, verbose: int = ...,
                 dry_run: int = ...) -> str: ...



from typing import Any, Mapping, Optional
from tkinter import Widget

DIALOG_ICON: str

class Dialog(Widget):
    widgetName: str = ...
    num: int = ...
    def __init__(self, master: Optional[Any] = ..., cnf: Mapping[str, Any] = ..., **kw) -> None: ...
    def destroy(self) -> None: ...



from typing import Any, Optional

XHTML_NAMESPACE: Any

def format_iso8601(obj): ...

class AtomFeed:
    default_generator: Any
    title: Any
    title_type: Any
    url: Any
    feed_url: Any
    id: Any
    updated: Any
    author: Any
    icon: Any
    logo: Any
    rights: Any
    rights_type: Any
    subtitle: Any
    subtitle_type: Any
    generator: Any
    links: Any
    entries: Any
    def __init__(self, title: Optional[Any] = ..., entries: Optional[Any] = ..., **kwargs): ...
    def add(self, *args, **kwargs): ...
    def generate(self): ...
    def to_string(self): ...
    def get_response(self): ...
    def __call__(self, environ, start_response): ...

class FeedEntry:
    title: Any
    title_type: Any
    content: Any
    content_type: Any
    url: Any
    id: Any
    updated: Any
    summary: Any
    summary_type: Any
    author: Any
    published: Any
    rights: Any
    links: Any
    categories: Any
    xml_base: Any
    def __init__(self, title: Optional[Any] = ..., content: Optional[Any] = ..., feed_url: Optional[Any] = ..., **kwargs): ...
    def generate(self): ...
    def to_string(self): ...



#OSC stands for Open Sound Control. It works with software synths like Reactor. This is a test file for it.
Đ_OSC_ip='192.168.1.104'
Đ_OSC_port=12345
#
_OSC_client=None  # This is a singleton
_OSC_values={}
def OSC_output(address,value):
    address=str(address)
    if not address[0] == '/':
        address='/' + address
    global Đ_OSC_ip
    global _OSC_client
    if not _OSC_client:
        _OSC_client=SimpleUDPClient(address=Đ_OSC_ip,port=Đ_OSC_port)
    _OSC_client.send_message(address=address,value=value)
    _OSC_values[address]=value  # Attempt to keep track of them (though it might sometimes drift out of sync etc idk i haven't tested it as of writing this)
def OSC_jiggle(address):
    address=str(address)
    if address in _OSC_values:
        original_value=_OSC_values[address]
    from time import sleep
    OSC_output(address,1)
    sleep(.1)
    OSC_output(address,0)
    sleep(.1)
    if address in _OSC_values:
        # noinspection PyUnboundLocalVariable
        OSC_output(address,original_value)
# Less abstract things lie below...
# Less abstract things lie below...
# Less abstract things lie below...
# Less abstract things lie below...
# Less abstract things lie below...
"""Client to send OSC datagrams to an OSC server via UDP."""
from collections import Iterable
import socket

"""Build OSC messages for client applications."""

"""Functions to get OSC types from datagrams and vice versa"""

import decimal
import struct

"""Parsing and conversion of NTP dates contained in datagrams."""

import datetime
import struct
import time
"""Representation of an OSC message in a pythonesque way."""

import logging

class ParseError(Exception):
    """Base exception raised when a datagram parsing error occurs."""

class OscMessage(object):
    """Representation of a parsed datagram representing an OSC message.

    An OSC message consists of an OSC Address Pattern followed by an OSC
    Type Tag String followed by zero or more OSC Arguments.
    """

    def __init__(self,dgram):
        self._dgram=dgram
        self._parameters=[]
        self._parse_datagram()

    def _parse_datagram(self):
        try:
            self._address_regexp,index=get_string(self._dgram,0)
            if not self._dgram[index:]:
                # No params is legit, just return now.
                return

            # Get the parameters types.
            type_tag,index=get_string(self._dgram,index)
            if type_tag.startswith(','):
                type_tag=type_tag[1:]

            # Parse each parameter given its type.
            for param in type_tag:
                if param == "i":  # Integer.
                    val,index=get_int(self._dgram,index)
                elif param == "f":  # Float.
                    val,index=get_float(self._dgram,index)
                elif param == "s":  # String.
                    val,index=get_string(self._dgram,index)
                elif param == "b":  # Blob.
                    val,index=get_blob(self._dgram,index)
                elif param == "r":  # RGBA.
                    val,index=get_rgba(self._dgram,index)
                elif param == "T":  # True.
                    val=True
                elif param == "F":  # False.
                    val=False
                # TODO: Support more exotic types as described in the specification.
                else:
                    logging.warning('Unhandled parameter type: {0}'.format(param))
                    continue
                self._parameters.append(val)
        except ParseError as pe:
            raise ParseError('Found incorrect datagram, ignoring it',pe)

    @property
    def address(self):
        """Returns the OSC address regular expression."""
        return self._address_regexp

    @staticmethod
    def dgram_is_message(dgram):
        """Returns whether this datagram starts as an OSC message."""
        return dgram.startswith(b'/')

    @property
    def size(self):
        """Returns the length of the datagram for this message."""
        return len(self._dgram)

    @property
    def dgram(self):
        """Returns the datagram from which this message was built."""
        return self._dgram

    @property
    def params(self):
        """Convenience method for list(self) to get the list of parameters."""
        return list(self)

    def __iter__(self):
        """Returns an iterator over the parameters of this message."""
        return iter(self._parameters)

# conversion factor for fractional seconds (maximum value of fractional part)
FRACTIONAL_CONVERSION=2 ** 32

# 63 zero bits followed by a one in the least signifigant bit is a special
# case meaning "immediately."
IMMEDIATELY=struct.pack('>q',1)

# From NTP lib.
_SYSTEM_EPOCH=datetime.date(*time.gmtime(0)[0:3])
_NTP_EPOCH=datetime.date(1900,1,1)
# _NTP_DELTA is 2208988800
_NTP_DELTA=(_SYSTEM_EPOCH - _NTP_EPOCH).days * 24 * 3600

class NtpError(Exception):
    """Base class for ntp module errors."""

def ntp_to_system_time(date):
    """Convert a NTP time to system time.

    System time is reprensented by seconds since the epoch in UTC.
    """
    return date - _NTP_DELTA

def system_time_to_ntp(date):
    """Convert a system time to NTP time.

    System time is reprensented by seconds since the epoch in UTC.
    """
    try:
        num_secs=int(date)
    except ValueError as e:
        raise NtpError(e)

    num_secs_ntp=num_secs + _NTP_DELTA

    sec_frac=float(date - num_secs)

    picos=int(sec_frac * FRACTIONAL_CONVERSION)

    return struct.pack('>I',int(num_secs_ntp)) + struct.pack('>I',picos)

class ParseError(Exception):
    """Base exception for when a datagram parsing error occurs."""

class BuildError(Exception):
    """Base exception for when a datagram building error occurs."""

# Constant for special ntp datagram sequences that represent an immediate time.
IMMEDIATELY=0

# Datagram length in bytes for types that have a fixed size.
_INT_DGRAM_LEN=4
_FLOAT_DGRAM_LEN=4
_DATE_DGRAM_LEN=_INT_DGRAM_LEN * 2
# Strings and blob dgram length is always a multiple of 4 bytes.
_STRING_DGRAM_PAD=4
_BLOB_DGRAM_PAD=4

def write_string(val):
    """Returns the OSC string equivalent of the given python string.

    Raises:
      - BuildError if the string could not be encoded.
    """
    try:
        dgram=val.encode('utf-8')  # Default, but better be explicit.
    except (UnicodeEncodeError,AttributeError) as e:
        raise BuildError('Incorrect string, could not encode {}'.format(e))
    diff=_STRING_DGRAM_PAD - (len(dgram) % _STRING_DGRAM_PAD)
    dgram+=(b'\x00' * diff)
    return dgram

def get_string(dgram,start_index):
    """Get a python string from the datagram, starting at pos start_index.

    According to the specifications, a string is:
    "A sequence of non-null ASCII characters followed by a null,
    followed by 0-3 additional null characters to make the total number
    of bits a multiple of 32".

    Args:
      dgram: A datagram packet.
      start_index: An index where the string starts in the datagram.

    Returns:
      A tuple containing the string and the new end index.

    Raises:
      ParseError if the datagram could not be parsed.
    """
    offset=0
    try:
        while dgram[start_index + offset] != 0:
            offset+=1
        if offset == 0:
            raise ParseError(
                'OSC string cannot begin with a null byte: %s' % dgram[start_index:])
        # Align to a byte word.
        if (offset) % _STRING_DGRAM_PAD == 0:
            offset+=_STRING_DGRAM_PAD
        else:
            offset+=(-offset % _STRING_DGRAM_PAD)
        # Python slices do not raise an IndexError past the last index,
        # do it ourselves.
        if offset > len(dgram[start_index:]):
            raise ParseError('Datagram is too short')
        data_str=dgram[start_index:start_index + offset]
        return data_str.replace(b'\x00',b'').decode('utf-8'),start_index + offset
    except IndexError as ie:
        raise ParseError('Could not parse datagram %s' % ie)
    except TypeError as te:
        raise ParseError('Could not parse datagram %s' % te)

def write_int(val):
    """Returns the datagram for the given integer parameter value

    Raises:
      - BuildError if the int could not be converted.
    """
    try:
        return struct.pack('>i',val)
    except struct.error as e:
        raise BuildError('Wrong argument value passed: {}'.format(e))

def get_int(dgram,start_index):
    """Get a 32-bit big-endian two's complement integer from the datagram.

    Args:
      dgram: A datagram packet.
      start_index: An index where the integer starts in the datagram.

    Returns:
      A tuple containing the integer and the new end index.

    Raises:
      ParseError if the datagram could not be parsed.
    """
    try:
        if len(dgram[start_index:]) < _INT_DGRAM_LEN:
            raise ParseError('Datagram is too short')
        return (
            struct.unpack('>i',
                          dgram[start_index:start_index + _INT_DGRAM_LEN])[0],
            start_index + _INT_DGRAM_LEN)
    except (struct.error,TypeError) as e:
        raise ParseError('Could not parse datagram %s' % e)

def write_float(val):
    """Returns the datagram for the given float parameter value

    Raises:
      - BuildError if the float could not be converted.
    """
    try:
        return struct.pack('>f',val)
    except struct.error as e:
        raise BuildError('Wrong argument value passed: {}'.format(e))

def get_float(dgram,start_index):
    """Get a 32-bit big-endian IEEE 754 floating point number from the datagram.

    Args:
      dgram: A datagram packet.
      start_index: An index where the float starts in the datagram.

    Returns:
      A tuple containing the float and the new end index.

    Raises:
      ParseError if the datagram could not be parsed.
    """
    try:
        if len(dgram[start_index:]) < _FLOAT_DGRAM_LEN:
            # Noticed that Reaktor doesn't send the last bunch of \x00 needed to make
            # the float representation complete in some cases, thus we pad here to
            # account for that.
            dgram=dgram + b'\x00' * (_FLOAT_DGRAM_LEN - len(dgram[start_index:]))
        return (
            struct.unpack('>f',
                          dgram[start_index:start_index + _FLOAT_DGRAM_LEN])[0],
            start_index + _FLOAT_DGRAM_LEN)
    except (struct.error,TypeError) as e:
        raise ParseError('Could not parse datagram %s' % e)

def get_blob(dgram,start_index):
    """ Get a blob from the datagram.

    According to the specifications, a blob is made of
    "an int32 size count, followed by that many 8-bit bytes of arbitrary
    binary data, followed by 0-3 additional zero bytes to make the total
    number of bits a multiple of 32".

    Args:
      dgram: A datagram packet.
      start_index: An index where the float starts in the datagram.

    Returns:
      A tuple containing the blob and the new end index.

    Raises:
      ParseError if the datagram could not be parsed.
    """
    size,int_offset=get_int(dgram,start_index)
    # Make the size a multiple of 32 bits.
    total_size=size + (-size % _BLOB_DGRAM_PAD)
    end_index=int_offset + size
    if end_index - start_index > len(dgram[start_index:]):
        raise ParseError('Datagram is too short.')
    return dgram[int_offset:int_offset + size],int_offset + total_size

def write_blob(val):
    """Returns the datagram for the given blob parameter value.

    Raises:
      - BuildError if the value was empty or if its size didn't fit an OSC int.
    """
    if not val:
        raise BuildError('Blob value cannot be empty')
    dgram=write_int(len(val))
    dgram+=val
    while len(dgram) % _BLOB_DGRAM_PAD != 0:
        dgram+=b'\x00'
    return dgram

def get_date(dgram,start_index):
    """Get a 64-bit big-endian fixed-point time tag as a date from the datagram.

    According to the specifications, a date is represented as is:
    "the first 32 bits specify the number of seconds since midnight on
    January 1, 1900, and the last 32 bits specify fractional parts of a second
    to a precision of about 200 picoseconds".

    Args:
      dgram: A datagram packet.
      start_index: An index where the date starts in the datagram.

    Returns:
      A tuple containing the system date and the new end index.
      returns osc_immediately (0) if the corresponding OSC sequence was found.

    Raises:
      ParseError if the datagram could not be parsed.
    """
    # Check for the special case first.
    if dgram[start_index:start_index + _DATE_DGRAM_LEN] == IMMEDIATELY:
        return IMMEDIATELY,start_index + _DATE_DGRAM_LEN
    if len(dgram[start_index:]) < _DATE_DGRAM_LEN:
        raise ParseError('Datagram is too short')
    num_secs,start_index=get_int(dgram,start_index)
    fraction,start_index=get_int(dgram,start_index)
    # Sum seconds and fraction of second:
    system_time=num_secs + (fraction / FRACTIONAL_CONVERSION)
    return ntp_to_system_time(system_time),start_index

def write_date(system_time):
    if system_time == IMMEDIATELY:
        return IMMEDIATELY

    try:
        return system_time_to_ntp(system_time)
    except NtpError as ntpe:
        raise BuildError(ntpe)

def write_rgba(val):
    """Returns the datagram for the given rgba32 parameter value

    Raises:
      - BuildError if the int could not be converted.
    """
    try:
        return struct.pack('>I',val)
    except struct.error as e:
        raise BuildError('Wrong argument value passed: {}'.format(e))

def get_rgba(dgram,start_index):
    """Get an rgba32 integer from the datagram.

    Args:
      dgram: A datagram packet.
      start_index: An index where the integer starts in the datagram.

    Returns:
      A tuple containing the integer and the new end index.

    Raises:
      ParseError if the datagram could not be parsed.
    """
    try:
        if len(dgram[start_index:]) < _INT_DGRAM_LEN:
            raise ParseError('Datagram is too short')
        return (
            struct.unpack('>I',
                          dgram[start_index:start_index + _INT_DGRAM_LEN])[0],
            start_index + _INT_DGRAM_LEN)
    except (struct.error,TypeError) as e:
        raise ParseError('Could not parse datagram %s' % e)

class BuildError(Exception):
    """Error raised when an incomplete message is trying to be built."""

class OscMessageBuilder(object):
    """Builds arbitrary OscMessage instances."""

    ARG_TYPE_FLOAT="f"
    ARG_TYPE_INT="i"
    ARG_TYPE_STRING="s"
    ARG_TYPE_BLOB="b"
    ARG_TYPE_RGBA="r"
    ARG_TYPE_TRUE="T"
    ARG_TYPE_FALSE="F"

    _SUPPORTED_ARG_TYPES=(
        ARG_TYPE_FLOAT,ARG_TYPE_INT,ARG_TYPE_BLOB,ARG_TYPE_STRING,ARG_TYPE_RGBA,ARG_TYPE_TRUE,ARG_TYPE_FALSE)

    def __init__(self,address=None):
        """Initialize a new builder for a message.

        Args:
          - address: The osc address to send this message to.
        """
        self._address=address
        self._args=[]

    @property
    def address(self):
        """Returns the OSC address this message will be sent to."""
        return self._address

    @address.setter
    def address(self,value):
        """Sets the OSC address this message will be sent to."""
        self._address=value

    @property
    def args(self):
        """Returns the (type, value) arguments list of this message."""
        return self._args

    def add_arg(self,arg_value,arg_type=None):
        """Add a typed argument to this message.

        Args:
          - arg_value: The corresponding value for the argument.
          - arg_type: A value in ARG_TYPE_* defined in this class,
                      if none then the type will be guessed.
        Raises:
          - ValueError: if the type is not supported.
        """
        if arg_type and arg_type not in self._SUPPORTED_ARG_TYPES:
            raise ValueError(
                'arg_type must be one of {}'.format(self._SUPPORTED_ARG_TYPES))
        if not arg_type:
            if isinstance(arg_value,str):
                arg_type=self.ARG_TYPE_STRING
            elif isinstance(arg_value,bytes):
                arg_type=self.ARG_TYPE_BLOB
            elif isinstance(arg_value,int):
                arg_type=self.ARG_TYPE_INT
            elif isinstance(arg_value,float):
                arg_type=self.ARG_TYPE_FLOAT
            elif arg_value == True:
                arg_type=self.ARG_TYPE_TRUE
            elif arg_value == False:
                arg_type=self.ARG_TYPE_FALSE
            else:
                raise ValueError('Infered arg_value type is not supported')
        self._args.append((arg_type,arg_value))

    def build(self):
        """Builds an OscMessage from the current state of this builder.

        Raises:
          - BuildError: if the message could not be build or if the address
                        was empty.

        Returns:
          - an OscMessage instance.
        """
        if not self._address:
            raise BuildError('OSC addresses cannot be empty')
        dgram=b''
        try:
            # Write the address.
            dgram+=write_string(self._address)
            if not self._args:
                dgram+=write_string(',')
                return OscMessage(dgram)

            # Write the parameters.
            arg_types="".join([arg[0] for arg in self._args])
            dgram+=write_string(',' + arg_types)
            for arg_type,value in self._args:
                if arg_type == self.ARG_TYPE_STRING:
                    dgram+=write_string(value)
                elif arg_type == self.ARG_TYPE_INT:
                    dgram+=write_int(value)
                elif arg_type == self.ARG_TYPE_FLOAT:
                    dgram+=write_float(value)
                elif arg_type == self.ARG_TYPE_BLOB:
                    dgram+=write_blob(value)
                elif arg_type == self.ARG_TYPE_RGBA:
                    dgram+=write_rgba(value)
                elif arg_type == self.ARG_TYPE_TRUE or arg_type == self.ARG_TYPE_FALSE:
                    continue
                else:
                    raise BuildError('Incorrect parameter type found {}'.format(
                        arg_type))

            return OscMessage(dgram)
        except BuildError as be:
            raise BuildError('Could not build the message: {}'.format(be))

class UDPClient(object):
    """OSC client to send OscMessages or OscBundles via UDP."""

    def __init__(self,address,port):
        """Initialize the client.

        As this is UDP it will not actually make any attempt to connect to the
        given server at ip:port until the send() method is called.
        """
        self._sock=socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
        self._sock.setblocking(0)
        self._address=address
        self._port=port

    def send(self,content):
        """Sends an OscBundle or OscMessage to the server."""
        self._sock.sendto(content.dgram,(self._address,self._port))

class SimpleUDPClient(UDPClient):
    """Simple OSC client with a `send_message` method."""

    def send_message(self,address,value):
        """Compose an OSC message and send it."""
        builder=OscMessageBuilder(address=address)
        if not isinstance(value,Iterable) or isinstance(value,(str,bytes)):
            values=[value]
        else:
            values=value
        for val in values:
            builder.add_arg(val)
        msg=builder.build()
        self.send(msg)



from tkinter.commondialog import Dialog
from typing import Any, Optional

ERROR: str
INFO: str
QUESTION: str
WARNING: str
ABORTRETRYIGNORE: str
OK: str
OKCANCEL: str
RETRYCANCEL: str
YESNO: str
YESNOCANCEL: str
ABORT: str
RETRY: str
IGNORE: str
CANCEL: str
YES: str
NO: str

class Message(Dialog):
    command: str = ...

def showinfo(title: Optional[str] = ..., message: Optional[str] = ..., **options: Any) -> str: ...
def showwarning(title: Optional[str] = ..., message: Optional[str] = ..., **options: Any) -> str: ...
def showerror(title: Optional[str] = ..., message: Optional[str] = ..., **options: Any) -> str: ...
def askquestion(title: Optional[str] = ..., message: Optional[str] = ..., **options: Any) -> str: ...
def askokcancel(title: Optional[str] = ..., message: Optional[str] = ..., **options: Any) -> bool: ...
def askyesno(title: Optional[str] = ..., message: Optional[str] = ..., **options: Any) -> bool: ...
def askyesnocancel(title: Optional[str] = ..., message: Optional[str] = ..., **options: Any) -> Optional[bool]: ...
def askretrycancel(title: Optional[str] = ..., message: Optional[str] = ..., **options: Any) -> bool: ...



# Stubs for errno

from typing import Mapping
import sys

errorcode: Mapping[int, str]

EPERM: int
ENOENT: int
ESRCH: int
EINTR: int
EIO: int
ENXIO: int
E2BIG: int
ENOEXEC: int
EBADF: int
ECHILD: int
EAGAIN: int
ENOMEM: int
EACCES: int
EFAULT: int
ENOTBLK: int
EBUSY: int
EEXIST: int
EXDEV: int
ENODEV: int
ENOTDIR: int
EISDIR: int
EINVAL: int
ENFILE: int
EMFILE: int
ENOTTY: int
ETXTBSY: int
EFBIG: int
ENOSPC: int
ESPIPE: int
EROFS: int
EMLINK: int
EPIPE: int
EDOM: int
ERANGE: int
EDEADLCK: int
ENAMETOOLONG: int
ENOLCK: int
ENOSYS: int
ENOTEMPTY: int
ELOOP: int
EWOULDBLOCK: int
ENOMSG: int
EIDRM: int
ECHRNG: int
EL2NSYNC: int
EL3HLT: int
EL3RST: int
ELNRNG: int
EUNATCH: int
ENOCSI: int
EL2HLT: int
EBADE: int
EBADR: int
EXFULL: int
ENOANO: int
EBADRQC: int
EBADSLT: int
EDEADLOCK: int
EBFONT: int
ENOSTR: int
ENODATA: int
ETIME: int
ENOSR: int
ENONET: int
ENOPKG: int
EREMOTE: int
ENOLINK: int
EADV: int
ESRMNT: int
ECOMM: int
EPROTO: int
EMULTIHOP: int
EDOTDOT: int
EBADMSG: int
EOVERFLOW: int
ENOTUNIQ: int
EBADFD: int
EREMCHG: int
ELIBACC: int
ELIBBAD: int
ELIBSCN: int
ELIBMAX: int
ELIBEXEC: int
EILSEQ: int
ERESTART: int
ESTRPIPE: int
EUSERS: int
ENOTSOCK: int
EDESTADDRREQ: int
EMSGSIZE: int
EPROTOTYPE: int
ENOPROTOOPT: int
EPROTONOSUPPORT: int
ESOCKTNOSUPPORT: int
ENOTSUP: int
EOPNOTSUPP: int
EPFNOSUPPORT: int
EAFNOSUPPORT: int
EADDRINUSE: int
EADDRNOTAVAIL: int
ENETDOWN: int
ENETUNREACH: int
ENETRESET: int
ECONNABORTED: int
ECONNRESET: int
ENOBUFS: int
EISCONN: int
ENOTCONN: int
ESHUTDOWN: int
ETOOMANYREFS: int
ETIMEDOUT: int
ECONNREFUSED: int
EHOSTDOWN: int
EHOSTUNREACH: int
EALREADY: int
EINPROGRESS: int
ESTALE: int
EUCLEAN: int
ENOTNAM: int
ENAVAIL: int
EISNAM: int
EREMOTEIO: int
EDQUOT: int



import sys
from abc import abstractmethod
from typing import Any, Optional, TypeVar, Union, Pattern, Text, Tuple

_T = TypeVar('_T', bound='Version')

class Version:
    def __repr__(self) -> str: ...

    if sys.version_info >= (3,):
        def __eq__(self, other: object) -> bool: ...
        def __lt__(self: _T, other: Union[_T, str]) -> bool: ...
        def __le__(self: _T, other: Union[_T, str]) -> bool: ...
        def __gt__(self: _T, other: Union[_T, str]) -> bool: ...
        def __ge__(self: _T, other: Union[_T, str]) -> bool: ...

    @abstractmethod
    def __init__(self, vstring: Optional[Text] = ...) -> None: ...
    @abstractmethod
    def parse(self: _T, vstring: Text) -> _T: ...
    @abstractmethod
    def __str__(self) -> str: ...
    if sys.version_info >= (3,):
        @abstractmethod
        def _cmp(self: _T, other: Union[_T, str]) -> bool: ...
    else:
        @abstractmethod
        def __cmp__(self: _T, other: Union[_T, str]) -> bool: ...

class StrictVersion(Version):
    version_re: Pattern[str]
    version: Tuple[int, int, int]
    prerelease: Optional[Tuple[Text, int]]

    def __init__(self, vstring: Optional[Text] = ...) -> None: ...
    def parse(self: _T, vstring: Text) -> _T: ...
    def __str__(self) -> str: ...
    if sys.version_info >= (3,):
        def _cmp(self: _T, other: Union[_T, str]) -> bool: ...
    else:
        def __cmp__(self: _T, other: Union[_T, str]) -> bool: ...

class LooseVersion(Version):
    component_re: Pattern[str]
    vstring: Text
    version: Tuple[Union[Text, int], ...]

    def __init__(self, vstring: Optional[Text] = ...) -> None: ...
    def parse(self: _T, vstring: Text) -> _T: ...
    def __str__(self) -> str: ...
    if sys.version_info >= (3,):
        def _cmp(self: _T, other: Union[_T, str]) -> bool: ...
    else:
        def __cmp__(self: _T, other: Union[_T, str]) -> bool: ...



# Source: https://hg.python.org/cpython/file/2.7/Lib/mutex.py

from collections import deque
from typing import Any, Callable, TypeVar

_ArgType = TypeVar('_ArgType')

class mutex:
    locked: bool
    queue: deque
    def __init__(self) -> None: ...
    def test(self) -> bool: ...
    def testandset(self) -> bool: ...
    def lock(self, function: Callable[[_ArgType], Any], argument: _ArgType) -> None: ...
    def unlock(self) -> None: ...
